#include <stdio.h>
#include <stdlib.h>

#include <GL/glew.h>
#include <GL/glut.h>

#include "textfile.h"


GLuint v,f,p;
float lpos[4] = {1,0.5,1,0};
float delta_x = 0.0;
float delta_y = 0.0;

void changeSize(int w, int h) 
{
  
  // Prevent a divide by zero, when window is too short (you can't make a window of zero width).
  if (h == 0)
    h = 1;
  
  float ratio = 1.0 * w / h;
  
  // Reset the coordinate system before modifying
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  
  // Set the viewport to be the entire window
  glViewport(0, 0, w, h);
  
  // Set the correct perspective.
  gluPerspective(45,ratio,1,1000);
  glMatrixMode(GL_MODELVIEW);
  
}

float a = 0;

void renderScene(void) 
{
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
  
  glLoadIdentity();
  gluLookAt(0.0,0.0,5.0, 
	    0.0,0.0,-1.0,
	    0.0f,1.0f,0.0f);
  
  glLightfv(GL_LIGHT0, GL_POSITION, lpos);
  glRotatef(a,0,1,1);
  glutSolidTeapot(1);
  a+=0.1;
  
  glutSwapBuffers();
}

void keyboard(unsigned char key, int x, int y) 
{
  if (key == 'q') 
    exit(0);
  else if (key == 'r') {
    glUniform3f(glGetUniformLocation(p, "Color"), 1.0, 0.0, 0.0);
  }
  else if (key == 'g') {
    glUniform3f(glGetUniformLocation(p, "Color"), 0.0, 1.0, 0.0);
  }
  else if (key == 'b') {
    glUniform3f(glGetUniformLocation(p, "Color"), 0.0, 0.0, 1.0);
  }
  else if (key == 'w') {
    delta_y += 0.1;
    glUniform2f(glGetUniformLocation(p, "Delta"), delta_x, delta_y);
  }
  else if (key == 's') {
    delta_y -= 0.1;
    glUniform2f(glGetUniformLocation(p, "Delta"), delta_x, delta_y);
  }
  else if (key == 'a') {
    delta_x -= 0.1;
    glUniform2f(glGetUniformLocation(p, "Delta"), delta_x, delta_y);
  }
  else if (key == 'd') {
    delta_x += 0.1;
    glUniform2f(glGetUniformLocation(p, "Delta"), delta_x, delta_y);
  }
}

#define printOpenGLError() printOglError(__FILE__, __LINE__)

// Returns 1 if an OpenGL error occurred, 0 otherwise.

int printOglError(char *file, int line)
{
  GLenum glErr;
  int    retCode = 0;
  
  glErr = glGetError();
  while (glErr != GL_NO_ERROR)
    {
      printf("glError in file %s @ line %d: %s\n", file, line, gluErrorString(glErr));
      retCode = 1;
      glErr = glGetError();
    }
  return retCode;
}


void printShaderInfoLog(GLuint obj)
{
  int infologLength = 0;
  int charsWritten  = 0;
  char *infoLog;
  
  glGetShaderiv(obj, GL_INFO_LOG_LENGTH,&infologLength);
  
  if (infologLength > 0) {
    infoLog = (char *)malloc(infologLength);
    glGetShaderInfoLog(obj, infologLength, &charsWritten, infoLog);
    printf("%s\n",infoLog);
    free(infoLog);
  }
}

void printProgramInfoLog(GLuint obj)
{
  int infologLength = 0;
  int charsWritten  = 0;
  char *infoLog;
  
  glGetProgramiv(obj, GL_INFO_LOG_LENGTH,&infologLength);
  
  if (infologLength > 0) {
    infoLog = (char *)malloc(infologLength);
    glGetProgramInfoLog(obj, infologLength, &charsWritten, infoLog);
    printf("%s\n",infoLog);
    free(infoLog);
  }
}

void setShaders() 
{
  char *vs = NULL,*fs = NULL,*fs2 = NULL;
  
  v = glCreateShader(GL_VERTEX_SHADER);
  f = glCreateShader(GL_FRAGMENT_SHADER);
  
  vs = textFileRead("uniform.vert");
  fs = textFileRead("uniform.frag");
  
  const char * vv = vs;
  const char * ff = fs;
  
  glShaderSource(v, 1, &vv,NULL);
  glShaderSource(f, 1, &ff,NULL);
  
  free(vs);free(fs);
  
  glCompileShader(v);
  glCompileShader(f);
  
  printShaderInfoLog(v);
  printShaderInfoLog(f);
  
  p = glCreateProgram();
  glAttachShader(p,v);
  glAttachShader(p,f);
  
  glLinkProgram(p);
  printProgramInfoLog(p);
  
  glUseProgram(p);

}

int main(int argc, char **argv) 
{
  glutInit(&argc, argv);
  glutInitDisplayMode(GLUT_DEPTH | GLUT_DOUBLE | GLUT_RGBA);
  glutInitWindowPosition(100,100);
  glutInitWindowSize(320,320);
  glutCreateWindow("Simple");
  
  glutDisplayFunc(renderScene);
  glutIdleFunc(renderScene);
  glutReshapeFunc(changeSize);
  glutKeyboardFunc(keyboard);
  
  glEnable(GL_DEPTH_TEST);
  glClearColor(1.0,1.0,1.0,1.0);
  glEnable(GL_CULL_FACE);
  
  glewInit();
  if (glewIsSupported("GL_VERSION_2_0"))
    printf("Ready for OpenGL 2.0\n");
  else {
    printf("OpenGL 2.0 not supported\n");
    exit(1);
  }
  
  setShaders();
  
  glutMainLoop();
  
  return 0;
}

